package fi.jyu.intellij.comtest.idea

import com.intellij.openapi.vfs.VirtualFile
import comtest.v1.ComTest
import comtest.v1.StringUtilities
import comtest.v1.Strings


internal object ComTestRunner {
    fun generateForJavaFile(file: VirtualFile): String? {
        // NOTE: This ComTest class is from ComTest1.jar which is the original ComTest library.
        //       The original is used for Java because the v2 does not support all of v1's features.
        return runComtest(
            file, ComTestSettings(
                outDir = "test",
                packagePostfix = ".test"
            )
        )
    }

    class ComTestSettings(val outDir: String? = null, val packagePostfix: String? = null)

    private val defaultSettings = ComTestSettings()

    private fun runComtest(file: VirtualFile, settings: ComTestSettings = defaultSettings): String? {
        val ext = file.extension
        if (ext != "java") {
            return null
        }
        val contents = StringUtilities.getFileContents(file.path) ?: return null
        if (contents.size < 1) {
            return null
        }

        // ComTestScanner's ctor and some options are private, so we use reflection to modify them for now
        val scanner =
            ComTest.ComTestScanner::class.java.getDeclaredConstructor(Strings::class.java).apply { isAccessible = true }
                .newInstance(contents)

        ComTest.ComTestScanner::class.java.getDeclaredField("directory").apply {
            isAccessible = true
            set(scanner, settings.outDir)
        }
        ComTest.ComTestScanner::class.java.getDeclaredField("packageName").apply {
            isAccessible = true
            set(scanner, settings.packagePostfix)
        }

        return scanner.scan(file.path)
    }
}