package fi.jyu.intellij.comtest.idea.util

import com.intellij.execution.testframework.AbstractJavaTestConfigurationProducer
import com.intellij.ide.projectView.impl.nodes.PsiDirectoryNode
import com.intellij.openapi.actionSystem.AnActionEvent
import com.intellij.openapi.actionSystem.CommonDataKeys
import com.intellij.openapi.actionSystem.PlatformCoreDataKeys
import com.intellij.openapi.project.Project
import com.intellij.psi.PsiClass
import com.intellij.psi.PsiElement
import com.intellij.psi.PsiJavaFile
import com.intellij.psi.PsiPackage
import com.intellij.psi.search.GlobalSearchScope

fun AnActionEvent.getValidComTestTarget(): PsiElement? {
    val el = this.getData(CommonDataKeys.PSI_FILE)
    if (el is PsiJavaFile) return el

    val selection = this.getData(PlatformCoreDataKeys.SELECTED_ITEMS) ?: return null
    val selected = selection.firstOrNull().let { it as? PsiDirectoryNode }?.value ?: return null
    val pkg = AbstractJavaTestConfigurationProducer.checkPackage(selected)
    return if (pkg != null) selected else null
}

fun PsiPackage.findAllClasses(project: Project, result: MutableList<PsiClass> = mutableListOf()): List<PsiClass> {
    result.addAll(this.getClasses(GlobalSearchScope.projectScope(project)))
    this.getSubPackages(GlobalSearchScope.projectScope(project)).forEach { it.findAllClasses(project, result) }
    return result
}