package fi.jyu.intellij.comtest.rider

import com.intellij.execution.filters.TextConsoleBuilderFactory
import com.intellij.execution.ui.ConsoleView
import com.intellij.execution.ui.ConsoleViewContentType
import com.intellij.openapi.project.Project
import com.intellij.openapi.wm.ToolWindow
import com.intellij.openapi.wm.ToolWindowAnchor
import com.intellij.openapi.wm.ToolWindowManager
import comtest.v2.processors.Logger

internal object ComTestLoggerWindow {

    @Suppress("UnstableApiUsage")
    private fun getToolWindow(proj: Project): ToolWindow {
        val manager = ToolWindowManager.getInstance(proj)
        var toolWindow = manager.getToolWindow("ComTest")
        if (toolWindow != null) {
            return toolWindow
        }

        toolWindow = manager.registerToolWindow("ComTest") {
            anchor = ToolWindowAnchor.BOTTOM
            shouldBeAvailable = true
            canCloseContent = false
            hideOnEmptyContent = true
        }

        val consoleView = TextConsoleBuilderFactory.getInstance().createBuilder(proj).console
        val content = toolWindow.contentManager.factory.createContent(consoleView.component, "ComTest", false)
        toolWindow.contentManager.addContent(content)
        return toolWindow
    }

    private fun getConsoleView(proj: Project): ConsoleView {
        val toolWindow = getToolWindow(proj)
        return toolWindow.contentManager.getContent(0)?.component as ConsoleView
    }

    object ComTestLogger : Logger {
        private var indentLevel = 0
        private var proj: Project? = null

        fun reset(proj: Project) {
            indentLevel = 0
            ComTestLogger.proj = proj
        }

        override fun openBlock() {
            indentLevel++
            proj?.let {
                getConsoleView(it).print("{\n", ConsoleViewContentType.NORMAL_OUTPUT)
            }
        }

        override fun closeBlock() {
            indentLevel--
            proj?.let {
                getConsoleView(it).print("}\n", ConsoleViewContentType.NORMAL_OUTPUT)
            }
        }

        override fun log(str: String?) {
            proj?.let {
                getConsoleView(it).print("${" ".repeat(indentLevel)}$str\n", ConsoleViewContentType.NORMAL_OUTPUT)
            }
        }

        override fun logPartial(str: String?) {
            proj?.let {
                getConsoleView(it).print("${" ".repeat(indentLevel)}$str", ConsoleViewContentType.NORMAL_OUTPUT)
            }
        }
    }

    fun open(proj: Project) {
        getToolWindow(proj).show()
        getConsoleView(proj).clear()
        ComTestLogger.reset(proj)
    }
}