package fi.jyu.intellij.comtest.rider

import com.intellij.openapi.project.Project
import comtest.v2.csharp.ComTestNUnitBackend
import comtest.v2.processors.*
import fi.jyu.intellij.comtest.common.util.Notifier
import fi.jyu.intellij.comtest.rider.util.riderSolutionFile

internal object ComTestRunner {
    init {
        val backend = ComTestNUnitBackend()
        FileProcessor.register(Language.Csharp, CsFileProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpCore, CsFileProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpProject, CsProjectProcessor(backend, ComTestLoggerWindow.ComTestLogger))
        FileProcessor.register(Language.CsharpSolution, CsSolutionProcessor(backend, ComTestLoggerWindow.ComTestLogger))
    }

    fun generateForDotNetSln(proj: Project) {
        ComTestLoggerWindow.open(proj)

        val slnFile = proj.riderSolutionFile
        if (slnFile == null) {
            Notifier.notifyError(proj, "No solution found or solution is virtual (virtual solutions are not supported). Load a solution to generate tests.")
            return
        }

        val slnEntry = FileEntry(slnFile.path, Language.CsharpSolution)
        FileProcessor.process(slnEntry, slnFile.parent, false)
    }
}